﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Threading;
using SingletonApp.Properties;

namespace SingletonApp
{
    static class Program
    {
        //该函数设置由不同线程产生的窗口的显示状态;
        [DllImport("user32.dll")]
        private static extern bool ShowWindowAsync(IntPtr hWnd, int nCmdShow);

        //判断窗口是否已最小化
        [DllImport("user32.dll")]
        private static extern bool IsIconic(IntPtr hWnd);

        private const int SW_SHOW = 5;//激活窗口的命令代码
        private const int SW_RESTORE = 9;//从最小化恢复时的命令代码

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            bool initiallyOwned = true;//是否应拥有互斥体的初始所有权
            bool bCreatedNew = false;//是否被授予互斥体的初始所有权

            //初始化互斥体
            Mutex m = new Mutex(initiallyOwned, "TTAPasswordBox", out bCreatedNew);

            //如果为false,则程序已经启动
            if (!(initiallyOwned && bCreatedNew))
            {
                m.Close();

                //获取窗口的句柄
                Settings s = new Settings();
                IntPtr i = (IntPtr)s.CurrentIntPtr;

                if (IsIconic(i))//如果窗口是最小化状态，激活并显示窗口
                {
                    ShowWindowAsync(i, SW_RESTORE);
                }
                else//如果不是最小化状态，弹出提示，并设置窗口
                {
                    //这个提示不是必要的
                    MessageBox.Show(new WindowWrapper(i), "程序已经运行，请勿重复启动！", "提示", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    ShowWindowAsync(i, SW_SHOW);
                }
            }
            else//可以启动程序
            {
                Application.EnableVisualStyles();
                Application.SetCompatibleTextRenderingDefault(false);

                Form1 login = new Form1();
                Application.Run(new Form1());

                //释放互斥体
                m.ReleaseMutex();
            }
        }
    }

    /// <summary>
    /// 包装一个窗口，公开Win32句柄
    /// </summary>
    public class WindowWrapper : System.Windows.Forms.IWin32Window
    {
        public WindowWrapper(IntPtr handle)
        {
            _hwnd = handle;
        }

        public IntPtr Handle
        {
            get { return _hwnd; }
        }

        private IntPtr _hwnd;
    }
}
